// Functions for initialization ADC and reading the knobs with them.
// Wiring: AN4 to X potentiometer, AN5 to Y potentiometer.

void initADC(void){
  CloseADC10();

  // Parameters discussed in HW #3 and report
  // Briefly: Sample/conversion timed automatically, manually start conversions,
  // set everything up on pin AN4 and AN5
  #define PARAM1    ADC_FORMAT_INTG16 | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_OFF 
  #define PARAM2    ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE|ADC_SCAN_OFF| \
                    ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF
  #define PARAM3    ADC_CONV_CLK_PB | ADC_SAMPLE_TIME_5 | ADC_CONV_CLK_Tcy2
  #define PARAM4    ENABLE_AN4_ANA | ENABLE_AN5_ANA
  #define PARAM5	SKIP_SCAN_ALL

  // Use ground as neg ref for A | use AN4 for input A
  SetChanADC10(ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN4 ); 
  OpenADC10( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 );
  
  EnableADC10();
}

unsigned short pot_read = 512;

// === Potentiometer Reading Thread ===
static PT_THREAD (protothread_adc(struct pt *pt))
{
    PT_BEGIN(pt);
    while (1){
        // Read the X knob and convert
        SetChanADC10(ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN4);
        PT_YIELD_TIME_msec(10);     // Allow mux to settled
        AcquireADC10();             // Trigger the next sample/conversion
        PT_YIELD_TIME_msec(10);     // Allow sample to complete
        pot_read = ReadADC10(0);    // Read the result of conversion
        s_x = 70 + ((signed short)pot_read - 512)/32;
        
        // Read the Y knob and convert
        SetChanADC10(ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN5);
        PT_YIELD_TIME_msec(10);
        AcquireADC10();            
        PT_YIELD_TIME_msec(10);
        pot_read = ReadADC10(0); 
        s_y = 70 + ((signed short)pot_read - 512)/32;
        
        PT_YIELD_TIME_msec(SCALE_T);
    }
    PT_END(pt);
}