// Header file contains GUI initialization and update thread.

void init_gui(void){
    
    // Groundwork
    tft_init_hw();
    tft_begin();
    tft_fillScreen(BKG_COL);
    tft_setRotation(3);
    
    // Divisions
    tft_fillRect(1, TOP_DIV, 320, 2, DIV_COL);
    tft_fillRect(1, BOT_DIV, 320, 2, DIV_COL);
    tft_fillRect(159, TOP_DIV, 2, 240, DIV_COL);
    tft_fillRect(79, BOT_DIV, 2, (320-BOT_DIV), DIV_COL);
    tft_fillRect(239, BOT_DIV, 2, (320-BOT_DIV), DIV_COL);
    
    // Top RPM readout
    tft_setTextColor2(TRPM_COL, BKG_COL);
    tft_setTextSize(TRPM_SIZE);
    tft_setCursor(TRPM_LBL_X, TRPM_Y);
    tft_writeString("RPM");
    
    // Channel Labels
    tft_setTextColor2(LBL_COL, BKG_COL);
    tft_setTextSize(CHAN_SIZE);
    tft_setCursor(CHANX_X, CHAN_Y);
    tft_writeString(CHANX_LBL);
    tft_setCursor(CHANY_X, CHAN_Y);
    tft_writeString(CHANY_LBL);
    
    tft_setCursor(PEAKX_X, PEAK_Y);
    tft_writeString(PEAK_LBL);
    tft_setCursor(NOWX_X, PEAK_Y);
    tft_writeString(NOW_LBL);
    tft_setCursor(PEAKY_X, PEAK_Y);
    tft_writeString(PEAK_LBL);
    tft_setCursor(NOWY_X, PEAK_Y);
    tft_writeString(NOW_LBL);
    
    // Scale label
    tft_setTextSize(S_SIZE);
    tft_setCursor(SX_X, S_Y);
    tft_writeString(S_LBL);
    tft_setCursor(SY_X, S_Y);
    tft_writeString(S_LBL);
    
    // Unit Labels
    tft_setTextColor2(DATA_COL, BKG_COL);
    tft_setTextSize(UNIT_SIZE);
    
    tft_setCursor(PUNITSX_X, RPM_Y);
    tft_writeString("RPM");
    tft_setCursor(PUNITSX_X, AMP_Y);
    tft_writeString("um");
    tft_setCursor(PUNITSX_X, PHASE_Y);
    tft_writeString("deg");
    
    tft_setCursor(NUNITSX_X, RPM_Y);
    tft_writeString("RPM");
    tft_setCursor(NUNITSX_X, AMP_Y);
    tft_writeString("um");
    tft_setCursor(NUNITSX_X, PHASE_Y);
    tft_writeString("deg");
    
    tft_setCursor(PUNITSY_X, RPM_Y);
    tft_writeString("RPM");
    tft_setCursor(PUNITSY_X, AMP_Y);
    tft_writeString("um");
    tft_setCursor(PUNITSY_X, PHASE_Y);
    tft_writeString("deg");
    
    tft_setCursor(NUNITSY_X, RPM_Y);
    tft_writeString("RPM");
    tft_setCursor(NUNITSY_X, AMP_Y);
    tft_writeString("um");
    tft_setCursor(NUNITSY_X, PHASE_Y);
    tft_writeString("deg");

}

// Periodically report measurements to screen. Also, heartbeat.
unsigned char heart_counter = 0;
unsigned char circ_state = 0;
static PT_THREAD (protothread_report(struct pt *pt))
{
    PT_BEGIN(pt);
    while(1) {
        PT_YIELD_TIME_msec(REFRESH_T);
        
        // Top RPM update
        tft_setTextColor2(TRPM_COL, BKG_COL);
        tft_setTextSize(TRPM_SIZE);
        tft_setCursor(TRPM_X, TRPM_Y);
        if (rpm > 0){
            sprintf(buffer, "%5.0f", rpm);
            tft_writeString(buffer);
        } else {
            tft_writeString("  LOW");
        }
        
        // Scale display
        tft_setTextColor2(LBL_COL, BKG_COL);
        tft_setTextSize(S_SIZE);
        tft_setCursor(SX_X_D, S_Y);
        sprintf(buffer, "%3.1f", s_x/10.0);
        tft_writeString(buffer);
        tft_setCursor(SY_X_D, S_Y);
        sprintf(buffer, "%3.1f", s_y/10.0);
        tft_writeString(buffer);
        
        // Signal status update
        tft_setTextSize(STAT_SIZE);
        tft_setCursor(STATX_X, STAT_Y);
        if (amp_x > 10) {
            tft_setTextColor2(STAT_GOOD_COL, BKG_COL);
            tft_writeString(STAT_GOOD);
        } else {
            tft_setTextColor2(STAT_BAD_COL, BKG_COL);
            tft_writeString(STAT_BAD);
        }
        tft_setCursor(STATY_X, STAT_Y);
        if (amp_y > 10) {
            tft_setTextColor2(STAT_GOOD_COL, BKG_COL);
            tft_writeString(STAT_GOOD);
        } else {
            tft_setTextColor2(STAT_BAD_COL, BKG_COL);
            tft_writeString(STAT_BAD);
        }
        
        // Null vector status update
        tft_setTextSize(NULL_SIZE);
        tft_setCursor(NULLX_X, NULL_Y);
        if (x_null) {
            tft_setTextColor2(NULL_GOOD_COL, BKG_COL);
            tft_writeString(NULL_GOOD);
        } else {
            tft_setTextColor2(NULL_BAD_COL, BKG_COL);
            tft_writeString(NULL_BAD);
        }
        tft_setCursor(NULLY_X, NULL_Y);
        if (y_null) {
            tft_setTextColor2(NULL_GOOD_COL, BKG_COL);
            tft_writeString(NULL_GOOD);
        } else {
            tft_setTextColor2(STAT_BAD_COL, BKG_COL);
            tft_writeString(NULL_BAD);
        }
        
        // Data update (12 fields! TODO: optimize for peak.)
        tft_setTextColor2(DATA_COL, BKG_COL);
        tft_setTextSize(DATA_SIZE);

        tft_setCursor(PDATAX_X, RPM_Y);
        sprintf(buffer, "%4.0f", p_rpm_x);
        tft_writeString(buffer);
        tft_setCursor(PDATAX_X, AMP_Y);
        sprintf(buffer, "%4.0f", p_amp_x);
        tft_writeString(buffer);
        tft_setCursor(PDATAX_X, PHASE_Y);
        sprintf(buffer, "%4.0f", p_phs_x);
        tft_writeString(buffer);

        tft_setCursor(NDATAX_X, RPM_Y);
        sprintf(buffer, "%4.0f", rpm);
        tft_writeString(buffer);
        tft_setCursor(NDATAX_X, AMP_Y);
        sprintf(buffer, "%4.0f", amp_x);
        tft_writeString(buffer);
        tft_setCursor(NDATAX_X, PHASE_Y);
        sprintf(buffer, "%4.0f", phs_x);
        tft_writeString(buffer);

        tft_setCursor(PDATAY_X, RPM_Y);
        sprintf(buffer, "%4.0f", p_rpm_y);
        tft_writeString(buffer);
        tft_setCursor(PDATAY_X, AMP_Y);
        sprintf(buffer, "%4.0f", p_amp_y);
        tft_writeString(buffer);
        tft_setCursor(PDATAY_X, PHASE_Y);
        sprintf(buffer, "%4.0f", p_phs_y);
        tft_writeString(buffer);

        tft_setCursor(NDATAY_X, RPM_Y);
        sprintf(buffer, "%4.0f", rpm);
        tft_writeString(buffer);
        tft_setCursor(NDATAY_X, AMP_Y);
        sprintf(buffer, "%4.0f", amp_y);
        tft_writeString(buffer);
        tft_setCursor(NDATAY_X, PHASE_Y);
        sprintf(buffer, "%4.0f", phs_y);
        tft_writeString(buffer);
        
        // Heartbeat update
        heart_counter++;
        if (heart_counter >= HEART_N){
            if (circ_state < 1){
                tft_fillCircle(HEART_X, HEART_Y, HEART_SIZE, ILI9340_GREEN);
                circ_state = 1;
            } else {
                tft_fillCircle(HEART_X, HEART_Y, HEART_SIZE, ILI9340_BLACK);
                circ_state = 0;
            }
            heart_counter = 0;
        }
        
        // Cheesy debounce
        accepting_press = 1;
    }
    PT_END(pt);
} 