/* 
 * File:   tool_macros.h
 * Author: Anthony McNicoll
 * Compiled from macros provided by Bruce Land/Syed Tahmid Mahbub
 * Created on September 23, 2015, 11:44 PM
 */

/* Data type reference:
 
 * Integer types come signed or unsigned:
 char:          8  bits
 short:         16 bits
 int, long:     32 bits
 long long:     64 bits
 
 * Floating-point types... are floating point:
 float, double: 32 bit precision
 long double:   64 bit precision
 */

// PORT B pullup macros
#define EnablePullDownB(bits) CNPUBCLR=bits; CNPDBSET=bits;
#define DisablePullDownB(bits) CNPDBCLR=bits;
#define EnablePullUpB(bits) CNPDBCLR=bits; CNPUBSET=bits;
#define DisablePullUpB(bits) CNPUBCLR=bits;

// PORT A pullup macros
#define EnablePullDownA(bits) CNPUACLR=bits; CNPDASET=bits;
#define DisablePullDownA(bits) CNPDACLR=bits;
#define EnablePullUpA(bits) CNPDACLR=bits; CNPUASET=bits;
#define DisablePullUpA(bits) CNPUACLR=bits;

// 16-bit fixed point type (+/- 32768 with 0.000015 resolution)
typedef signed int fix16 ;
#define multfix16(a,b) ((fix16)(((( signed long long)(a))*(( signed long long)(b)))>>16)) //multiply two fixed 16:16
#define float2fix16(a) ((fix16)((a)*65536.0)) // 2^16
#define fix2float16(a) ((float)(a)/65536.0)
#define fix2int16(a)    ((int)((a)>>16))
#define int2fix16(a)    ((fix16)((a)<<16))
#define divfix16(a,b) ((fix16)((((signed long long)(a)<<16)/(b)))) 
#define sqrtfix16(a) (float2fix16(sqrt(fix2float16(a)))) 
#define absfix16(a) abs(a)

#define sign(a) ((a > 0) - (a < 0))